import React, { useEffect, useState } from 'react';
import styles from './inputs.module.scss';
import { DropdownTrigger, DropdownContainer, DropdownItem, DropdownItemContainer } from '../dropdowns';
import Info from '../info';
import { Tooltip, TooltipContent, TooltipProvider, TooltipTrigger } from '../tooltip';
import { Button } from '../buttons';

type SliderProps = {
  label: string;
  type: 'slider';
  value: number;
  min: number;
  max: number;
  step: number;
  showValue?: boolean;
  unit?: string;
  tooltip?: string;
  onChange?: (value: number) => void;
};

type TextProps = {
  label?: string;
  type: 'text';
  size?: 'xs' | 'sm' | 'md' | 'xl';
  placeholder?: string;
  value: string;
  required?: boolean;
  errorText?: string;
  visible?: boolean;
  disabled?: boolean;
  tooltip?: string;
  inline?: boolean;
  info?: string;
  className?: string;
  validate?: (value: any) => boolean;
  onChange?: (value: string) => void;
  onClick?: (e: React.MouseEvent<HTMLElement>) => void;
};

type NumberProps = {
  label?: string;
  type: 'number';
  size?: 'xs' | 'sm' | 'md' | 'xl';
  variant?: 'primary' | 'secondary' | 'danger' | 'warning' | 'success' | 'info' | 'base';
  placeholder?: string;
  value: number;
  required?: boolean;
  errorText?: string;
  visible?: boolean;
  disabled?: boolean;
  tooltip?: string | React.ReactNode;
  info?: string;
  inline?: boolean;
  validate?: (value: any) => boolean;
  onChange?: (value: number) => void;
  onRevert?: (value: number) => void;
  onKeyDown?: (event: React.KeyboardEvent<HTMLInputElement>) => void;
  max?: number;
  min?: number;
  className?: string;
  containerClassName?: string;
  lazy?: boolean;
};

type CheckboxProps = {
  label?: string;
  type: 'checkbox';
  options: Array<string>;
  value: Array<string>;
  tooltip?: string;
  onChange?: (value: Array<string>) => void;
};

type BooleanProps = {
  label: string;
  type: 'boolean';
  options?: any;
  value: boolean;
  tooltip?: string;
  onChange?: (value: boolean) => void;
  size?: 'xs' | 'sm' | 'md' | 'xl';
  info?: string;
  required?: boolean;
  className?: string;
  inline?: boolean;
};

type ToggleSwitchProps = {
  label: string;
  type: 'toggle';
  value: boolean;
  classText?: string;
  tooltip?: string;
  onChange?: (value: boolean) => void;
};

type TimeProps = {
  label: string;
  type: 'time';
  value: string;
  tooltip?: string;
  onChange?: (value: string) => void;
  showTimezone: boolean;
};

type RadioProps = {
  label?: string;
  type: 'radio';
  options: Array<string>;
  value: string;
  tooltip?: string;
  onChange?: (value: string) => void;
};

type DropdownProps = {
  label?: string;
  overrideRender?: React.ReactNode;
  type: 'dropdown';
  size?: 'xs' | 'sm' | 'md' | 'xl';
  tooltip?: string | React.ReactNode;
  required?: boolean;
  inline?: boolean;
  buttonVariant?: 'primary' | 'outline' | 'ghost';
  info?: string;
  disabled?: boolean;
  className?: string;
  value?: number | string;
  options: number[] | string[] | { [key: string]: string }[];
  onChange?: (value: number | string) => void;
};

export type InputProps =
  | TextProps
  | SliderProps
  | CheckboxProps
  | DropdownProps
  | RadioProps
  | BooleanProps
  | NumberProps
  | TimeProps
  | ToggleSwitchProps;

export const Input = (props: InputProps) => {
  switch (props.type) {
    case 'slider':
      return <SliderInput {...(props as SliderProps)} />;
    case 'text':
      return <TextInput {...(props as TextProps)} />;
    case 'checkbox':
      return <CheckboxInput {...(props as CheckboxProps)} />;
    case 'dropdown':
      return <DropdownInput {...(props as DropdownProps)} />;
    case 'radio':
      return <RadioInput {...(props as RadioProps)} />;
    case 'boolean':
      return <BooleanInput {...(props as BooleanProps)} />;
    case 'number':
      return <NumberInput {...(props as NumberProps)} />;
    case 'time':
      return <TimeInput {...(props as TimeProps)} />;
    case 'toggle':
      return <ToggleSwitchInput {...(props as ToggleSwitchProps)} />;
    default:
      return null;
  }
};

export const SliderInput = ({ label, value, min, max, step, unit, showValue = true, onChange, tooltip }: SliderProps) => {
  return (
    <Tooltip>
      <TooltipTrigger>
        <div className={styles['slider']}>
          <label className="label flex flex-row justify-between items-end">
            <span className="label-text">{label}</span>
            {showValue ? (
              <div className="label-text">
                {value}
                {unit}
              </div>
            ) : null}
          </label>

          <input
            type="range"
            min={min}
            max={max}
            step={step}
            value={value}
            onChange={(e) => {
              if (onChange) {
                onChange(parseFloat(e.target.value));
              }
            }}
            aria-labelledby="input-slider"
          />
        </div>
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

export const TextInput = ({
  label = undefined,
  placeholder,
  value = '',
  size = 'md',
  required = false,
  visible = true,
  errorText,
  validate,
  disabled = false,
  onChange,
  onClick = () => {},
  inline = false,
  tooltip,
  info,
  className,
}: TextProps) => {
  const [isValid, setIsValid] = React.useState<boolean>(true);

  if (!tooltip && inline) tooltip = label;
  return (
    <Tooltip>
      <TooltipTrigger className={styles['input'] + ' form-control w-full' + (inline ? ' grid grid-cols-2 items-center gap-0.5' : '')}>
        {label && (
          <label className="label p-0">
            <span
              className={`text-sm font-medium text-secondary-700 ${required && "after:content-['*'] after:ml-0.5 after:text-danger-500"}`}
            >
              {label}
            </span>
            {required && isValid ? null : <span className="label-text-alt text-error">{errorText}</span>}
            {info && <Info tooltip={info} placement={'left'} />}
          </label>
        )}
        <input
          type={visible ? 'text' : 'password'}
          placeholder={placeholder}
          className={
            `${size} bg-light border border-secondary-200 placeholder-secondary-400 focus:outline-none block w-full focus:ring-1 ${
              isValid ? '' : 'input-error'
            }` + (className ? ` ${className}` : '')
          }
          value={value.toString()}
          onChange={(e) => {
            if (required && validate) {
              setIsValid(validate(e.target.value));
            }
            if (onChange) {
              onChange(e.target.value);
            }
          }}
          onClick={onClick}
          required={required}
          disabled={disabled}
        />
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

export const NumberInput = ({
  label,
  placeholder,
  size = 'md',
  value = 0,
  inline = false,
  required = false,
  visible = true,
  variant = 'base',
  errorText,
  validate,
  disabled = false,
  onChange,
  onRevert,
  tooltip,
  info,
  onKeyDown,
  max,
  min,
  className,
  containerClassName,
  lazy = false,
}: NumberProps) => {
  const [isValid, setIsValid] = React.useState<boolean>(true);
  const [inputValue, setInputValue] = useState<number>(value);

  useEffect(() => {
    setInputValue(value);
  }, [value]);

  if (!tooltip && inline && label) tooltip = label;
  return (
    <div className={styles['input'] + `${containerClassName ? ` ${containerClassName}` : ''}`}>
      <TooltipProvider delayDuration={50}>
        <Tooltip>
          <TooltipTrigger className={'form-control w-full' + (inline && label ? ' grid grid-cols-2 items-center gap-0.5' : '')}>
            {label && (
              <label className="label p-0">
                <span
                  className={`text-sm text-left truncate font-medium text-secondary-700 ${required && "after:content-['*'] after:ml-0.5 after:text-danger-500"}`}
                >
                  {label}
                </span>
                {required && isValid ? null : <span className="label-text-alt text-error">{errorText}</span>}
                {info && <Info tooltip={info} placement={'left'} />}
              </label>
            )}
            <div className="relative items-center">
              <input
                type="number"
                placeholder={placeholder}
                className={`${size} bg-light border border-secondary-200 placeholder-secondary-400 focus:outline-none block w-full sm:text-sm focus:ring-1 ${
                  isValid ? '' : 'input-error'
                }${className ? ` ${className}` : ''}`}
                value={inputValue.toString()}
                onChange={(e) => {
                  if (required && validate) {
                    setIsValid(validate(e.target.value));
                  }
                  setInputValue(Number(e.target.value));
                  if (!lazy && onChange) {
                    onChange(Number(e.target.value));
                  }
                }}
                required={required}
                disabled={disabled}
                onKeyDown={(e) => {
                  if (lazy && e.key === 'Enter') {
                    if (onChange) {
                      onChange(inputValue);
                    }
                  }
                  if (onKeyDown) onKeyDown(e);
                }}
                max={max}
                min={min}
              />
              {lazy && value !== inputValue && (
                <div className="absolute top-0 right-1 h-full flex flex-row items-center">
                  <Button
                    className="hover:bg-success-600 hover:text-white border-none m-0 p-0 h-fit"
                    variant="outline"
                    tooltip="Apply Change"
                    size={size}
                    rounded
                    iconComponent={'icon-[ic--round-check-circle-outline]'}
                    onClick={() => {
                      if (onChange) onChange(inputValue);
                    }}
                  ></Button>
                  <Button
                    className="hover:bg-warning-600 hover:text-white border-none m-0 p-0 h-fit"
                    variant="outline"
                    tooltip="Revert Change"
                    size={size}
                    rounded
                    iconComponent={'icon-[ic--outline-cancel]'}
                    onClick={() => {
                      if (onRevert) onRevert(inputValue);
                      setInputValue(value);
                    }}
                  ></Button>
                </div>
              )}
            </div>
          </TooltipTrigger>
          {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
        </Tooltip>
      </TooltipProvider>
    </div>
  );
};

export const RadioInput = ({ label, value, options, onChange, tooltip }: RadioProps) => {
  return (
    <Tooltip>
      <TooltipTrigger>
        <label className="label p-0">
          <span className="label-text">{label}</span>
        </label>
        {options.map((option, index) => (
          <label key={index} className="label cursor-pointer w-fit gap-2 px-0 py-1">
            <input
              type="radio"
              name={option}
              checked={value === option}
              onChange={() => {
                if (onChange) {
                  onChange(option);
                }
              }}
              className="radio radio-xs radio-primary"
            />
            <span className="label-text">{option}</span>
          </label>
        ))}
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

export const CheckboxInput = ({ label, value, options, onChange, tooltip }: CheckboxProps) => {
  return (
    <Tooltip>
      <TooltipTrigger>
        {label && (
          <label className="label p-0">
            <span className="label-text">{label}</span>
          </label>
        )}
        {options.map((option, index) => (
          <label key={index} className="label cursor-pointer w-fit gap-2 px-0 py-1">
            <input
              type="checkbox"
              name={option}
              checked={Array.isArray(value) && value.includes(option)}
              onClick={(event) => {
                if (event.ctrlKey) {
                  const updatedValue = (event.target as any).checked ? [option] : value.filter((val) => val !== option);
                  if (onChange) {
                    onChange(updatedValue);
                  }
                } else {
                  const updatedValue = (event.target as any).checked ? [...value, option] : value.filter((val) => val !== option);
                  if (onChange) {
                    onChange(updatedValue);
                  }
                }
              }}
              onChange={() => {
                // to remove warning
              }}
              className="checkbox checkbox-xs"
            />
            <span className="label-text">{option}</span>
          </label>
        ))}
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

export const BooleanInput = ({ label, value, onChange, tooltip, info, size, required, className }: BooleanProps) => {
  return (
    <Tooltip>
      <TooltipTrigger className={className + 'w-full flex justify-between'}>
        {label && (
          <label className="label p-0">
            <span
              className={`text-${size} text-left truncate font-medium text-secondary-700 ${required && "after:content-['*'] after:ml-0.5 after:text-danger-500"}`}
            >
              {label}
            </span>
            {info && <Info tooltip={info} />}
          </label>
        )}
        <input
          type="checkbox"
          checked={value}
          onChange={(event) => {
            if (onChange) {
              onChange(event.target.checked);
            }
          }}
          className="checkbox checkbox-xs"
          aria-label={`Toggle ${label}`}
        />
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

function parsedValue(item: number | string | { [key: string]: string }, key: boolean = false) {
  switch (typeof item) {
    case 'number':
      return item.toString();
    case 'object':
      return key ? Object.keys(item)[0] : Object.values(item)[0];
  }

  return item;
}

function currentValue(value: string | number | undefined, options?: { [key: string]: string }[]) {
  if (options != null && typeof options[0] == 'object') {
    return parsedValue(options.find((x) => x[value as string]) as { [key: string]: string });
  }

  return value;
}

export const DropdownInput = ({
  label,
  value,
  overrideRender,
  options,
  onChange,
  required = false,
  tooltip,
  size = 'sm',
  inline = true,
  disabled = false,
  buttonVariant = 'primary',
  className = '',
  info,
}: DropdownProps) => {
  const [isDropdownOpen, setIsDropdownOpen] = React.useState<boolean>(false);

  if (!tooltip && inline) tooltip = label;

  return (
    <Tooltip>
      <TooltipTrigger className={className + ' w-full' + (inline ? ' grid grid-cols-2 items-center gap-0.5' : '')}>
        {label && (
          <label className="label p-0">
            <span
              className={`text-${size} text-left truncate font-medium text-secondary-700 ${required && "after:content-['*'] after:ml-0.5 after:text-danger-500"}`}
            >
              {label}
            </span>
            {info && <Info tooltip={info} />}
          </label>
        )}
        <DropdownContainer
          open={isDropdownOpen}
          onOpenChange={(ref) => {
            setIsDropdownOpen(ref);
          }}
        >
          <DropdownTrigger
            variant={buttonVariant}
            title={overrideRender || currentValue(value, options as { [key: string]: string }[])}
            size={size}
            className="cursor-pointer"
            disabled={disabled}
            onClick={() => {
              setIsDropdownOpen(!isDropdownOpen);
            }}
          >
            {overrideRender}
          </DropdownTrigger>
          {isDropdownOpen && (
            <DropdownItemContainer className="w-fit">
              {options &&
                options.map((item: any, index: number) => (
                  <DropdownItem
                    key={index}
                    value={parsedValue(item)}
                    selected={value === item}
                    onClick={() => {
                      if (onChange) {
                        onChange(parsedValue(item, true));
                      }
                      setIsDropdownOpen(false);
                    }}
                  />
                ))}
            </DropdownItemContainer>
          )}
        </DropdownContainer>
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

export const TimeInput = ({ label, value, onChange, tooltip, showTimezone = true }: TimeProps) => {
  const hours = Array.from({ length: 24 }, (_, i) => i.toString().padStart(2, '0'));
  const minutes = Array.from({ length: 60 }, (_, i) => i.toString().padStart(2, '0'));
  const timezones = ['UTC', 'CET', 'EDT'];

  const splitValue = value.split(':');
  const hour = splitValue[0] || '00';
  const minute = splitValue[1] || '00';
  const timezone = splitValue[2] || 'UTC';

  const handleTimeChange = (newHour: string, newMinute: string) => {
    const updatedTime = `${newHour}:${newMinute}:${timezone}`;
    if (onChange) onChange(updatedTime);
  };

  const handleTimezoneChange = (newTimezone: string) => {
    const updatedTime = `${hour}:${minute}:${newTimezone}`;
    if (onChange) onChange(updatedTime);
  };

  return (
    <Tooltip>
      <TooltipTrigger>
        <label className={`label cursor-pointer w-fit gap-2 px-0 py-1`}>
          <span className="text-sm">{label}</span>
        </label>

        <div className="flex items-center gap-2">
          <select
            value={hour}
            onChange={(e) => handleTimeChange(e.target.value, minute)}
            className="select select-sm bg-secondary-0 hover:bg-secondary-100"
          >
            {hours.map((hourOption) => (
              <option key={hourOption} value={hourOption}>
                {hourOption}
              </option>
            ))}
          </select>
          <span>H</span>
          <span>:</span>

          <select value={minute} onChange={(e) => handleTimeChange(hour, e.target.value)} className="select select-sm">
            {minutes.map((minuteOption) => (
              <option key={minuteOption} value={minuteOption}>
                {minuteOption}
              </option>
            ))}
          </select>
          <span>M</span>
        </div>

        {showTimezone && (
          <div className="flex items-center gap-2 mt-2">
            <label className="text-sm">Timezone:</label>
            <select value={timezone} onChange={(e) => handleTimezoneChange(e.target.value)} className="select select-sm">
              {timezones.map((timezoneOption) => (
                <option key={timezoneOption} value={timezoneOption}>
                  {timezoneOption}
                </option>
              ))}
            </select>
          </div>
        )}
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};

export const ToggleSwitchInput = ({ label, classText, value, tooltip, onChange }: ToggleSwitchProps) => {
  const [isSelected, setIsSelected] = useState(value);

  const handleClick = () => {
    const newValue = !isSelected;
    setIsSelected(newValue);
    if (onChange) {
      onChange(newValue);
    }
  };
  return (
    <Tooltip>
      <TooltipTrigger>
        <div className="flex items-center space-x-2">
          <div
            className={`relative  flex w-10 h-5 rounded-full cursor-pointer transition-all duration-500 ${isSelected ? 'bg-secondary-800' : 'bg-secondary-300'}`}
            onClick={handleClick}
          >
            <div
              className={`absolute top-0 left-0 w-5 h-5 rounded-full items-center bg-white transition-all duration-500 shadow-lg border-1 border-gray ${isSelected ? 'translate-x-full' : 'translate-x-0'} `}
            ></div>
          </div>
          <span className={classText}>{label}</span>
        </div>
      </TooltipTrigger>
      {tooltip && <TooltipContent>{tooltip}</TooltipContent>}
    </Tooltip>
  );
};
