import { memo, useState, useMemo, useEffect } from 'react';
import { NodeAttribute, RelationNodeAttributes, SchemaReactflowRelationNode, toHandleId } from '../../../model';
import { useAppDispatch, useQuerybuilderGraph, useQuerybuilderSettings } from '@graphpolaris/shared/lib/data-access';
import { addWarning } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { setQuerybuilderGraphology, toQuerybuilderGraphology, attributeShownToggle } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import { PillAttributes } from '../../pillattributes/PillAttributes';
import { Button, DropdownContainer, DropdownTrigger, RelationPill, DropdownItemContainer, DropdownItem } from '@graphpolaris/shared/lib/components';
import { Icon } from '@graphpolaris/shared/lib/components/icon';
import { TextInput } from '@graphpolaris/shared/lib/components/inputs';
import { PopoverContext } from '@graphpolaris/shared/lib/components/layout/Popover';
import { Handle, Position } from 'reactflow';
import { pillWidth } from '@graphpolaris/shared/lib/components/pills/pill.const';
import { useQuerybuilderAttributesShown } from '@graphpolaris/shared/lib/data-access/store';
import { isEqual } from 'lodash-es';

export const QueryRelationPill = memo((node: SchemaReactflowRelationNode) => {
  const data = node.data;
  const graph = useQuerybuilderGraph();
  const graphologyGraph = useMemo(() => toQuerybuilderGraphology(graph), [graph]);
  const settings = useQuerybuilderSettings();
  const dispatch = useAppDispatch();
  const graphologyNodeAttributes = useMemo<RelationNodeAttributes | undefined>(
    () => (graphologyGraph.hasNode(node.id) ? { ...(graphologyGraph.getNodeAttributes(node.id) as RelationNodeAttributes) } : undefined),
    [node.id],
  );

  const IconMap = {
    temporal: 'icon-[ic--baseline-calendar-today]',
    spatial: 'icon-[ic--baseline-map]',
    numerical: 'icon-[ic--baseline-numbers]',
    categorical: 'icon-[ic--baseline-abc]',
    default: 'icon-[ic--baseline-place]',
  };

  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph],
  );
  const [openDropdown, setOpenDropdown] = useState(false);

  const [filter, setFilter] = useState<string>('');

  const [depth, setDepth] = useState<{ min: number; max: number }>({
    min: data.depth.min || settings.depth.min,
    max: data.depth.max || settings.depth.max,
  });

  // TODO: must do this once design is chosen
  // const [direction, setDirection] = useState<RelationshipHandleArrowType>('right');

  useEffect(() => {
    setDepth({ min: data.depth.min || settings.depth.min, max: data.depth.max || settings.depth.max });
  }, [data.depth]);

  const filteredAttributes = useMemo(() => {
    if (filter == null || filter.length == 0) return data.attributes;

    return data.attributes.filter((attr) => {
      return attr.handleData.attributeName?.toLocaleLowerCase().includes(filter.toLocaleLowerCase());
    });
  }, [filter]);

  const onNodeUpdated = () => {
    if (depth.min < 0) {
      dispatch(addWarning('The minimum depth cannot be smaller than 0'));
    } else if (depth.max > 99) {
      dispatch(addWarning('The maximum depth cannot be larger than 99'));
    } else if (depth.min > depth.max) {
      dispatch(addWarning('The minimum depth cannot be larger than the maximum depth'));
    } else {
      graphologyGraph.setNodeAttribute<any>(node.id, 'depth', depth);
      dispatch(setQuerybuilderGraphology(graphologyGraph));
    }
  };

  function removeNode() {
    graphologyGraph.dropNode(node.id);
    dispatch(setQuerybuilderGraphology(graphologyGraph));
  }

  function addAttribute(attribute: NodeAttribute) {
    dispatch(attributeShownToggle(attribute.handleData));
  }

  const attributesBeingShown = useQuerybuilderAttributesShown();
  function isAttributeAdded(attribute: NodeAttribute): boolean {
    return attributesBeingShown.some((x) => isEqual(x, attribute.handleData))
  }

  // TODO: must do this once design is chosen
  // const onChangeDirection = () => {
  //   if (direction === 'right') {
  //     setDirection('both');
  //     graphologyGraph.setNodeAttribute<any>(node.id, 'direction', 'both');
  //   } else {
  //     setDirection('right');
  //     graphologyGraph.setNodeAttribute<any>(node.id, 'direction', 'right');
  //   }
  //   dispatch(setQuerybuilderGraphology(graphologyGraph));
  // };

  return (
    <div className="w-fit h-fit p-3 bg-transparent nowheel">
      <RelationPill
        title={
          <div className="flex flex-row w-full">
            <span className="flex-grow text-justify truncate">{data?.name}</span>
            
            <DropdownContainer>
              <DropdownTrigger size="md">
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="2xs"
                  iconComponent={openDropdown ? 'icon-[ic--baseline-arrow-drop-up]' : 'icon-[ic--baseline-arrow-drop-down]'}
                  className={openDropdown ? 'border-secondary-200' : ''}
                />
              </DropdownTrigger>

              <DropdownItemContainer>
                <PopoverContext.Consumer>
                  {popover => <DropdownItem value={'Add/remove attribute'} onClick={(e) => {
                    popover?.setOpen(false);
                    setOpenDropdown(false);
                  }} submenu={
                    [
                      <TextInput 
                        type={'text'} 
                        placeholder="Filter" 
                        size="xs" 
                        className="mb-1 min-w-40 rounded-sm" 
                        value={filter}
                        onClick={(e) => e.stopPropagation()}
                        onChange={(v) => setFilter(v)} />,

                      filteredAttributes.map(attr => 
                        <DropdownItem
                          key={attr.handleData.attributeName + attr.handleData.nodeId}
                          value={attr.handleData.attributeName ?? ''} 
                          selected={isAttributeAdded(attr)}
                          onClick={(_) => addAttribute(attr)}>
                            <Icon component={attr?.handleData?.attributeDimension != null ? IconMap[attr.handleData.attributeDimension] : undefined} className="ms-2 float-end" size={16} />
                        </DropdownItem>
                      )
                    ]
                  } />}
                </PopoverContext.Consumer>
                <DropdownItem value="Remove" className="text-danger" onClick={(e) => removeNode()} />
                
              </DropdownItemContainer>
            </DropdownContainer>
          </div>
        }
        withHandles="horizontal"
        handleLeft={
          <Handle
            // onDoubleClick={onChangeDirection}
            id={toHandleId(data.leftEntityHandleId)}
            position={Position.Left}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !b-0 !right-0 !left-0'}
            type="target"
          ></Handle>
        }
        handleRight={
          <Handle
            // onDoubleClick={onChangeDirection}
            id={toHandleId(data.rightEntityHandleId)}
            position={Position.Right}
            className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !right-0 !left-0'}
            type="source"
          ></Handle>
        }
      >
        <PillAttributes
          node={node}
          attributes={data?.attributes || []}
          attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
          mr={-pillWidth * 0.05}
        />
      </RelationPill>
    </div>
  );
});

QueryRelationPill.displayName = 'RelationPill';
export default QueryRelationPill;
