import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { GraphMetadata } from '@graphpolaris/shared/lib/data-access/statistics';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import { select, selectAll } from 'd3';
import React, { useEffect, useMemo, useRef } from 'react';
import { VISComponentType, VisualizationPropTypes } from '../../common';
import ArcVisNodes from './components/arcplotd3nodes';

import { Edge, Node } from '@graphpolaris/shared/lib/data-access';
import { scaleOrdinal, schemeCategory10 } from 'd3';
import { ArcEdge, ArcNode } from './arcplotvis.types';
import ArcVisLine from './components/arcplotd3line';
import ArcVisLinks from './components/arcplotd3links';

export interface ArcVisProps {
  iconStyle: string;
  ordering: string;
  labels: boolean;
}

const configuration: ArcVisProps = {
  iconStyle: 'circle',
  labels: true,
  ordering: 'Name',
};

export const configStyle = {
  colorText: 'hsl(var(--clr-sec--800))',
  colorTextUnselect: 'hsl(var(--clr-sec--400))',
  colorLinesHyperEdge: 'hsl(var(--clr-black))',
  HOVER_TRANSITION_DURATION: 350,
};

const margin = { top: 20, right: 20, bottom: 20, left: 20 };

export const ArcPlotVis = React.memo(({ data, configuration }: VisualizationPropTypes) => {
  const svgRef = useRef<SVGSVGElement>(null);
  const [dimensions, setDimensions] = React.useState({ width: 0, height: 0 });

  useEffect(() => {
    const handleResize = () => {
      const width = svgRef.current?.clientWidth;
      const height = svgRef.current?.clientHeight;
      // console.log('width', width);
      // console.log('height', height);

      const dim = { width: width || 0, height: height || 0 };
      setDimensions(dim);
    };

    window.addEventListener('resize', handleResize);
    handleResize(); // Call the handleResize function initially

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  const getNodeOrdering = (node: Node) => {
    switch (configuration.ordering) {
      case 'Name':
        return node.label.charCodeAt(0) * 10;
      case 'Group':
        return margin.left + Math.random() * (dimensions.width - margin.right);
      case 'Frequency':
        return margin.left + Math.random() * (dimensions.width - margin.right);
      default:
        return margin.left + Math.random() * (dimensions.width - margin.right);
    }
  };

  const [dataNodes, dataLinks] = useMemo(() => {
    const color = scaleOrdinal(schemeCategory10).domain(data.nodes.map((node) => node.label));
    const colorScale = scaleOrdinal(schemeCategory10).domain(data.edges.map((edge) => edge.attributes.Type as string));

    const dataNodes = data.nodes.map((node, index) => {
      let x = getNodeOrdering(node);
      return {
        ...node,
        x: x,
        y: dimensions.height / 2,
        size: Math.random() * 10,
        fill: color(node.label),
        label: 'N' + index,
      } as ArcNode;
    }) as ArcNode[];

    const dataLinks = data.edges.map((edge: Edge) => {
      const fromNode = dataNodes.find((node: ArcNode) => node._id === edge.from);
      const toNode = dataNodes.find((node: ArcNode) => node._id === edge.to);
      // const fromNode = dataNodes.find((node: ArcNode) => node._id === edge.from);
      // const toNode = dataNodes.find((node: ArcNode) => node._id === edge.to);

      if (!fromNode || !toNode) {
        // throw new Error('Node not found; ArcEdge cannot be constructred');

        // alternatively, we could just (silently or not) skip the edge
        console.error('Node not found; ArcEdge cannot be constructred; Skipping this edge.');
        return null;
      }
      return {
        ...edge,
        x1: fromNode.x,
        x2: toNode.x,
        thickness: fromNode.size,
        fill: colorScale(edge.attributes['Type'] as string),
      } as ArcEdge;
    });

    return [dataNodes as ArcNode[], dataLinks as ArcEdge[]]; // MIB: no idea why typescript needs this cast so explicitly
  }, [data, configuration, dimensions]);

  const handleMouseEnterNodes = (event: React.MouseEvent<SVGGElement, MouseEvent>) => {
    const targetClassList = (event.currentTarget as SVGGElement).classList;
    const hoveredID = targetClassList[1].replace('node-', '');
    // all elements - unselect
    selectAll('.arcLabel').selectAll('text').attr('fill', configStyle.colorTextUnselect);
    selectAll('.arcLabel').selectAll('circle').attr('fill', configStyle.colorTextUnselect);
    selectAll('.arcLabel').selectAll('rect').attr('fill', configStyle.colorTextUnselect);
    selectAll('.arcLabel').selectAll('polygon').attr('fill', configStyle.colorTextUnselect);

    // select element - select hovered
    const targetElement = selectAll('.' + targetClassList[1]);
    targetElement.selectAll('text').attr('fill', configStyle.colorText).style('opacity', 1);
    targetElement.selectAll('circle').attr('fill', configStyle.colorText);
    targetElement.selectAll('rect').attr('fill', configStyle.colorText);
    targetElement.selectAll('polygon').attr('fill', configStyle.colorText);

    selectAll('.arcLinks').selectAll('path').attr('stroke', configStyle.colorTextUnselect).attr('stroke-opacity', 0.1);
    const selectionEdges = selectAll('.arcLinks');
    selectionEdges.each((d, i, nodes) => {
      const gElement = select(nodes[i]);
      const targetClassList = gElement.attr('class').split(' ');

      const from = targetClassList[1].replace('from-', '');
      const to = targetClassList[2].replace('to-', '');

      if (from === hoveredID || to === hoveredID) {
        gElement
          .selectAll('path')
          .transition()
          .duration(configStyle.HOVER_TRANSITION_DURATION)
          .attr('stroke', configStyle.colorLinesHyperEdge)
          .attr('stroke-opacity', 1);
      }
    });
  };

  const handleMouseOutNodes = (event: React.MouseEvent<SVGGElement, MouseEvent>) => {
    // all elements - unselect
    // selectAll('.arcLabel').selectAll('text').attr('fill', configStyle.colorText);

    const selectionLabels = selectAll('.arcLabel');
    selectionLabels.each((d, i, nodes) => {
      const gElement = select(nodes[i]);
      const targetClassList = gElement.attr('class').split(' ');

      // console.log('targetClassList', targetClassList);

      const node = dataNodes.find((node: ArcNode) => node._id === targetClassList[1].replace('node-', ''));
      if (!node) throw new Error('Node not found; ArcNode cannot be color-reset after hover');

      gElement.selectAll('circle').attr('fill', node.fill);
      gElement.selectAll('rect').attr('fill', node.fill);
      gElement.selectAll('polygon').attr('fill', node.fill);
    });

    const selectionEdges = selectAll('.arcLinks');
    selectionEdges.each((d, i, nodes) => {
      const gElement = select(nodes[i]);
      const targetClassList = gElement.attr('class').split(' ');

      const edge = findEdgeByIDs(targetClassList[1], targetClassList[2]);
      if (!edge) throw new Error('Edge not found; ArcEdge cannot be color-reset after hover ');
      gElement
        .selectAll('path')
        .transition()
        .duration(configStyle.HOVER_TRANSITION_DURATION)
        .attr('stroke', edge.fill)
        .attr('stroke-opacity', 0.2);
    });
  };

  const findEdgeByIDs = (from: string, to: string) => {
    return dataLinks.find((edge) => edge.from === from.replace('from-', '') && edge.to === to.replace('to-', ''));
  };

  return (
    <svg ref={svgRef} className="h-full w-full">
      {<ArcVisLine height={dimensions.height / 2} start={margin.left} end={dimensions.width - margin.right} />}
      {
        <ArcVisNodes
          nodes={dataNodes}
          handleMouseEnter={handleMouseEnterNodes}
          handleMouseOut={handleMouseOutNodes}
          iconStyle={configuration.iconStyle}
          drawLabels={configuration.labels}
        />
      }
      {
        <g transform={`translate(0, ${dimensions.height / 2})`}>
          <ArcVisLinks nodes={dataNodes} edges={dataLinks} height={dimensions.height} />
        </g>
      }
    </svg>
  );
});

const ArcPlotVisSettings = ({
  configuration,
  graph,
  updateSettings,
}: {
  configuration: ArcVisProps;
  graph: GraphMetadata;
  updateSettings: (val: any) => void;
}) => {
  return (
    <SettingsContainer>
      <Input
        type="dropdown"
        label="Select an Ordering"
        value={configuration.ordering}
        options={['Group', 'Frequency', 'Name']}
        onChange={(val) => updateSettings({ theme: val })}
      />
      <Input
        type="dropdown"
        label="Icon style"
        value={configuration.iconStyle}
        options={['circle', 'square', 'triangle']}
        onChange={(val) => updateSettings({ iconStyle: val })}
      />
      {/* <Input type="checkbox" label="Draw Labels" value={configuration.labels} onChange={(val) => updateSettings({ labels: val })} /> */}
    </SettingsContainer>
  );
};

export const ArcVisComponent: VISComponentType = {
  displayName: 'Arc Plot',
  component: ArcPlotVis,
  settings: ArcPlotVisSettings,
  configuration: configuration,
};
export default ArcVisComponent;
